--[[
DHUD4_Druid.lua
Copyright (c) 2006 by Markus Inger, 2006 by Caeryn Dryad, 2007-2010 by Horacio Hoyos

This file is part of DHUD4.

    DHUD4 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    DHUD4 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DHUD4.  If not, see <http://www.gnu.org/licenses/>.
]]
local DHUD4 = LibStub("AceAddon-3.0"):GetAddon("DHUD4")
local L = LibStub("AceLocale-3.0"):GetLocale("DHUD4")

local MODNAME = "DHUD4_Druid"
local DHUD4_Druid = DHUD4:NewModule(MODNAME, DHUD4.Abilities, "AceEvent-3.0")
local VERSION = tonumber(("$Rev: 3 $"):match("%d+"))

local db
local OptGetter, OptSetter, ColorGetter, ColorSetter
do
	local mod = DHUD4_Druid
	function OptGetter(info)
		local key = info[#info]
		return db[key]
	end

	function OptSetter(info, value)
		local key = info[#info]
		db[key] = value
		mod:Refresh()
	end

    function ColorGetter (info)
        local key1, key2 = info[#info-1], info[#info]
        return db[key1][key2].r, db[key1][key2].g, db[key1][key2].b
    end

    function ColorSetter (info, r, g, b)
        local key1, key2 = info[#info-1], info[#info]
        db[key1][key2].r = r
        db[key1][key2].g = g
        db[key1][key2].b = b
        mod:Refresh()
    end
end

--Local Options
local options
local function getOptions()
    if not options then
        --Options table
        options = DHUD4_Druid:GetOptions()
        -- Add function calls
        options.arg = MODNAME
        options.get = OptGetter
        options.set = OptSetter
        options.args.enabled.get = function() return DHUD4:GetModuleEnabled(MODNAME) end
        options.args.enabled.set = function(info, value) DHUD4:SetModuleEnabled(MODNAME, value) end
        options.args.general.disabled = function() return not DHUD4:GetModuleEnabled(MODNAME) end
        options.args.general.args.colors.get = ColorGetter
		options.args.general.args.colors.set = ColorSetter
        -- Add class specifics
    	options.args.class = {
            order = 4,
            type = 'group',
            name = L["Druid"],
            disabled = function() return not DHUD4:GetModuleEnabled(MODNAME); end,
            args = {
                combos = {
                    type = 'toggle',
                    order = 1,
                    name = L["Combos"],
                    desc = L["Track druid combo points"],
                    hidden = false,
                },
                lacerates = {
                    type = 'toggle',
                    order = 2,
                    name = L["Lacerates"],
                    desc = L["Track druid lacerates"],
                    hidden = false,
                },
                laceratesTimer = {
                    type = 'toggle',
                    order = 3,
                    name = L["Lacerates Timer"],
                    desc = L["Track Druid lacerates expiration"],
                    disabled = timerDisabled,
                    hidden = false,
                },
                lifeblooms = {
                    type = 'toggle',
                    order = 2,
                    name = L["Lifebloom"],
                    desc = L["Track druid lifebloom"],
                    hidden = false,
                },
                lifebloomsTimer = {
                    type = 'toggle',
                    order = 3,
                    name = L["Lifebloom Timer"],
                    desc = L["Track Druid lifebloom expiration"],
                    disabled = timerDisabled,
                    hidden = false,
                },
            },
        }
	end
	return options
end


function DHUD4_Druid:OnInitialize()
    
    local _, englishClass = UnitClass("player")
    if (englishClass == "DRUID") then
        self:SetEnabledState(DHUD4:GetModuleEnabled(MODNAME))
        self.defaults.profile.combos = true
        self.defaults.profile.lacerates = true
        self.defaults.profile.laceratesTimer  = true
        self.defaults.profile.lifeblooms  = true
        self.defaults.profile.lifebloomsTimer  = true
        self.db = DHUD4.db:RegisterNamespace(MODNAME, self.defaults)
        db = self.db.profile
        self.layout = false
        DHUD4_Druid:SetColors(db.colors)
        DHUD4:RegisterModuleOptions(MODNAME, getOptions, L["Druid Abilities"])
        self.side = db.side
    else
        self:SetEnabledState(false)
        DHUD4:SetModuleEnabled(MODNAME, false)
    end
    
end

function DHUD4_Druid:OnEnable()

    --DHUD4:Debug("DHUD4_Druid:OnEnable")
    if not self:IsEnabled() then return end
    if ( DHUD4:GetModuleEnabled("DHUD4_Auras") ) then
        local auras = DHUD4:GetModule("DHUD4_Auras")
        self.db.profile.side = (auras:GetSide() == "r") and "l" or "r"
    end
    self:Refresh()
    -- Listen to bar pops
	self:RegisterMessage("DHUD4_BAR")
end


function DHUD4_Druid:Refresh()

    --DHUD4:Debug(MODNAME, "Refresh")
    db = self.db.profile
    self.side = db.side
    self.scale = db.scale
    self.fontSize = db.fontSize
    self:_Refresh()
    if (self.layout) then
        self:SetLayout()
    else
        if UnitExists("target") then
            self:PLAYER_TARGET_CHANGED()
        else
            self:HideAll()
        end
        if ( db.combos or db.lacerates or db.lifeblooms) then
            self:RegisterEvent("PLAYER_TARGET_CHANGED")
            self:RegisterEvent("UPDATE_SHAPESHIFT_FORM")
            self:RegisterEvent("UNIT_AURA")
        end
        if ( db.lacerates or db.lifeblooms) then
            self:RegisterEvent("UNIT_AURA")
        end
        if ( db.combos ) then
            self:RegisterEvent("UNIT_COMBO_POINTS")
        end
    end
end

function DHUD4_Druid:UNIT_COMBO_POINTS(event, who)
	
	--DHUD4:Debug("Abilities UNIT_COMBO_POINTS");
	if ( who == "player" ) then
		self:UpdateCombos("player")
	end
end

function DHUD4_Druid:PLAYER_TARGET_CHANGED(event)
	
	--DHUD4:Debug("Abiities PLAYER_TARGET_CHANGED");
	if ( UnitExists("target") ) then
		if ( UnitInVehicle("player") and db.vehiCombos ) then
			self:UpdateCombos("player")
		else
            self:UPDATE_SHAPESHIFT_FORM()
        end
    else
        self:HideAll()
	end
end

function DHUD4_Druid:UPDATE_SHAPESHIFT_FORM(event, who)
	
	--DHUD4:Debug("Abilities UPDATE_SHAPESHIFT_FORM")
	local powerType = UnitPowerType("player")
	if ( db.lifeblooms and ( powerType == 0 ) ) then
		-- Is Druid
		self:UpdateBuff("Lifebloom", db.lifebloomsTimer, db.tips)
	elseif ( db.lacerates and ( powerType == 1 ) ) then
		-- Is Bear
        self:UpdateDebuff("Lacerate", db.laceratesTimer, db.tips)
	elseif ( db.druidCombos and ( powerType == 3) ) then
		-- Is Cat
		self:UpdateCombos("player")
	end
end

function DHUD4_Druid:UNIT_AURA(event, who)
    --DHUD4:Debug("Druid: UNIT_AURA", who)
	if who == "target" then
		local powerType = UnitPowerType("player")
        --DHUD4:Debug("Druid: UNIT_AURA", db.lifeblooms, powerType)
        if ( db.lifeblooms and ( powerType == 0 ) ) then
            -- Is Druid
            self:UpdateBuff("Lifebloom", db.lifebloomsTimer, db.tips)
        elseif ( db.lacerates and ( powerType == 1 ) ) then
            -- Is Bear
            self:UpdateDebuff("Lacerate", db.laceratesTimer, db.tips)
        end
    end
end

function DHUD4_Druid:SetLayout()

    self.layout = true
    self:UnregisterAllEvents()
    self:HideAll()
    if ( db.combos or db.lacerates or db.lifeblooms) then
        self:_SetLayout(db.combos and 5 or 3, db.lacerates or db.lifeblooms)
    end
end

function DHUD4_Druid:EndLayout()

    self.layout = false
    self:Refresh()
end